////////////////////////////////////////////////////////////////////////
/// \file CRTGeoObjectSorter.cxx
/// \brief Interface to algorithm class for sorting of AuxDetGeo objects
///
/// Ported from AuxDetGeoObjectSorterLArIAT.cxx (Author: brebel@fnal.gov)
///
/// \version $Id:  $
/// \author mastbaum@uchicago.edu
////////////////////////////////////////////////////////////////////////

#include "sbndcode/CRT/CRTGeoObjectSorter.h"
#include "larcorealg/Geometry/AuxDetGeo.h"
#include "larcorealg/Geometry/AuxDetSensitiveGeo.h"

namespace geo {

  //----------------------------------------------------------------------------
  // Define sort order for AuxDets in standard configuration
  static bool sortAuxDetSBND(const AuxDetGeo& ad1, const AuxDetGeo& ad2) {
    // Sort using the center of the detector - primary ordering by z,
    // then y, and x
    auto const c1 = ad1.GetCenter();
    auto const c2 = ad2.GetCenter();

    if (c1.Z() != c2.Z()) {
      return c1.Z() < c2.Z();
      }
    if (c1.Y() != c2.Y()) {
      return c1.Y() < c2.Y();
    }
    return c1.X() < c2.X();
  }

  //----------------------------------------------------------------------------
  // Define sort order for AuxDetSensitives in standard configuration
  static bool sortAuxDetSensitiveSBND(const AuxDetSensitiveGeo& ad1,
                                      const AuxDetSensitiveGeo& ad2)
  {
    // Sort using the center of the detector - primary ordering by z,
    // then y, and x
    auto const c1 = ad1.GetCenter();
    auto const c2 = ad2.GetCenter();

    if (c1.Z() != c2.Z()) {
      return c1.Z() < c2.Z();
      }
    if (c1.Y() != c2.Y()) {
      return c1.Y() < c2.Y();
    }
    return c1.X() < c2.X();
  }

  //----------------------------------------------------------------------------
  CRTGeoObjectSorter::CRTGeoObjectSorter(
      fhicl::ParameterSet const&) {}

  //----------------------------------------------------------------------------
  void CRTGeoObjectSorter::SortAuxDets(
      std::vector<geo::AuxDetGeo> & adgeo) const {
    std::sort(adgeo.begin(), adgeo.end(), sortAuxDetSBND);
  }

  //----------------------------------------------------------------------------
  void CRTGeoObjectSorter::SortAuxDetSensitive(
      std::vector<geo::AuxDetSensitiveGeo> & adsgeo) const {
    std::sort(adsgeo.begin(), adsgeo.end(), sortAuxDetSensitiveSBND);
  }

}  // namespace geo
